DefaultWorldMapZoom 				= {}
DefaultWorldMapZoom.title 			= "DefaultWorldMapZoom"
DefaultWorldMapZoom.name 			= "WorldMapZoom"
DefaultWorldMapZoom.version         = "v1.0.6"
DefaultWorldMapZoom.SettingsVersion = 1
DefaultWorldMapZoom.data 			= {}

DefaultWorldMapZoom.SettingsDefault = {
	ZoomLevel = 9,
	ZoomOutOfSubZone = false
}
	
-- LIB
DefaultWorldMapZoom.LAM = LibStub("LibAddonMenu-2.0")

-- GLOBALS
local DefaultWorldMapZoomLocalCallbackManager = CALLBACK_MANAGER

function DefaultWorldMapZoom.Initialize(eventCode, addOnName)
    if (addOnName ~= DefaultWorldMapZoom.name) then return end
	
	DefaultWorldMapZoom.data.firstTime = true
	DefaultWorldMapZoom.data.tileTextureLoaded = nil

    -- LOAD SAVED SETTINGS
    DefaultWorldMapZoom.Settings = ZO_SavedVars:NewAccountWide(
		"WorldMapZoomSettings",
		DefaultWorldMapZoom.SettingsVersion,
		"Settings",
		DefaultWorldMapZoom.SettingsDefaults,
		nil
    )
	
	if DefaultWorldMapZoom.Settings.ZoomLevel == nil then
		DefaultWorldMapZoom.Settings.ZoomLevel = DefaultWorldMapZoom.SettingsDefault.ZoomLevel
	end	
	
	if DefaultWorldMapZoom.Settings.ZoomOutOfSubZone == nil then
		DefaultWorldMapZoom.Settings.ZoomOutOfSubZone = DefaultWorldMapZoom.SettingsDefault.ZoomOutOfSubZone
	end	
	
    DefaultWorldMapZoom.BuildSettingsMenu()	
end

function DefaultWorldMapZoom.Zoom(zoomLevel)	
	if zoomLevel < 1 then zoomLevel = 1 end
	if zoomLevel > 9 then zoomLevel = 9 end
		
	-- No need to continue if you want it max zoomed in
	if zoomLevel == 9 then return end
		
	-- mimicking clicking on the minus button according to zoomLevel
	for i = zoomLevel + 1, 9 do
		zo_callLater(function() ZO_WorldMapZoomMinus_OnClicked() end, (i - zoomLevel) * 100)
	end	
	
	if DefaultWorldMapZoom.Settings.ZoomOutOfSubZone and GetMapType() == MAPTYPE_SUBZONE then
		zo_callLater(function() ZO_WorldMap_MouseUp(DefaultWorldMapZoom.Settings.ZoomLevel, 2, true) end, zoomLevel * 100)
	end
		
end

function DefaultWorldMapZoom.Start(layerIndex, activeLayerIndex)
	if ZO_WorldMap:IsHidden() then return end
	if DefaultWorldMapZoom.data.firstTime == false then return end
	DefaultWorldMapZoom.data.firstTime = false
		
	DefaultWorldMapZoom.Zoom(DefaultWorldMapZoom.Settings.ZoomLevel)
end

function DefaultWorldMapZoom.End(layerIndex, activeLayerIndex)
	if ZO_WorldMap:IsHidden() == false then DefaultWorldMapZoom.data.firstTime = true end
end

function DefaultWorldMapZoom.showZone(zoneName, subZoneName, newSubzone)
	if ZO_WorldMap:IsHidden() then return end
		
	-- Start checking every 100 miliseconds if the map tile changed to the new zone
	DefaultWorldMapZoomLocalCallbackManager:FireCallbacks("DefaultWorldMapOnZoneChanged", 1)
end

function DefaultWorldMapZoom.OnZoneChanged(scope)
	if scope == nil then scope = 1 end
	if scope == 500 then return end
	if DefaultWorldMapZoom.data.tileTexture == nil then 
		DefaultWorldMapZoom.data.tileTexture = GetMapTileTexture(1) 
	end
	
	if DefaultWorldMapZoom.data.tileTexture ~= GetMapTileTexture(1) then
		DefaultWorldMapZoom.data.tileTexture = nil
		
		--The new tile has been loaded
		if DefaultWorldMapZoom.Settings.ZoomOutOfSubZone and GetMapType() == MAPTYPE_SUBZONE then
			ZO_WorldMap_MouseUp(DefaultWorldMapZoom.Settings.ZoomLevel, 2, true)
		end
		
		zo_callLater(function() DefaultWorldMapZoom.Zoom(DefaultWorldMapZoom.Settings.ZoomLevel) end, 400)
		return
	else 
		--The new zone tiles have not been loaded yet, call the event again in 10 miliseconds
		local newScope = scope + 1
		zo_callLater(function() DefaultWorldMapZoomLocalCallbackManager:FireCallbacks("DefaultWorldMapOnZoneChanged", newScope) end, 10)
	end
end

function DefaultWorldMapZoom.disableZoom(nodeIndex)
	EVENT_MANAGER:UnregisterForEvent(DefaultWorldMapZoom.title, EVENT_ZONE_CHANGED)
	EVENT_MANAGER:UnregisterForEvent(DefaultWorldMapZoom.title, EVENT_ACTION_LAYER_PUSHED)
	EVENT_MANAGER:UnregisterForEvent(DefaultWorldMapZoom.title, EVENT_ACTION_LAYER_POPPED)
end

function DefaultWorldMapZoom.enableZoom(nodeIndex)
	EVENT_MANAGER:RegisterForEvent(DefaultWorldMapZoom.title, EVENT_ZONE_CHANGED, DefaultWorldMapZoom.showZone)
	EVENT_MANAGER:RegisterForEvent(DefaultWorldMapZoom.title, EVENT_ACTION_LAYER_PUSHED, DefaultWorldMapZoom.Start)
	EVENT_MANAGER:RegisterForEvent(DefaultWorldMapZoom.title, EVENT_ACTION_LAYER_POPPED, DefaultWorldMapZoom.End)
end

function DefaultWorldMapZoom.BuildSettingsMenu()
	DefaultWorldMapZoom.panelTable = {
		type = "panel",
		name = "World Map Zoom",
		displayName = "World Map Zoom",
		author = "Shawashirya (EU)",
		version = DefaultWorldMapZoom.version,
		slashCommand = "/wmz",
	}
	DefaultWorldMapZoom.optionsTable = {
		[1] = {
				type = "description",
				text = "The World Map Zoom addon will let you adjust the default settings of your World Map.",
				width = "full",	
		},
		[2] = {
			  type = "slider",
			  name = "Default Zoom level",
			  tooltip = "Default world map zoom level.",
			  min = 1,
			  max = 9,
			  getFunc = function() return DefaultWorldMapZoom.Settings.ZoomLevel end,
			  setFunc = function(value) DefaultWorldMapZoom.Settings.ZoomLevel = value end,
		},
		[3] = {
			type = "checkbox",
			name = "Show zone only",
			tooltip = "When this option is enabled and you move through a subzone and have the world map open. It will zoom out back to the zone.",
			getFunc = function() return DefaultWorldMapZoom.Settings.ZoomOutOfSubZone end, 
			setFunc = function(value) DefaultWorldMapZoom.Settings.ZoomOutOfSubZone = value end,
		},		 
	}
	
	DefaultWorldMapZoom.OptionsPanel = DefaultWorldMapZoom.LAM:RegisterAddonPanel(DefaultWorldMapZoom.name .. "Panel", DefaultWorldMapZoom.panelTable)	
	DefaultWorldMapZoom.LAM:RegisterOptionControls(DefaultWorldMapZoom.name .. "Panel", DefaultWorldMapZoom.optionsTable)
	
	--[[local panelID = DefaultWorldMapZoom.LAM:CreateControlPanel(DefaultWorldMapZoom.title .."PanelID", "World map")
	
	DefaultWorldMapZoom.LAM:AddHeader(panelID, DefaultWorldMapZoom.title .. "controlName", " ")
	
	DefaultWorldMapZoom.LAM:AddSlider(	
		panelID, 
		DefaultWorldMapZoom.title .."ZoomLevel",
		"Default Zoom level",
		"Default world map zoom level",
		1,
		9,
		1, 
		function() return DefaultWorldMapZoom.Settings.ZoomLevel end, 
		function(value) DefaultWorldMapZoom.Settings.ZoomLevel = value end,
		false, 
		""
	)
	
	DefaultWorldMapZoom.LAM:AddCheckbox(
		panelID,
		DefaultWorldMapZoom.title .."ZoomOutOfSubZone",
		"Show zone only",
		"When this option is enabled and you move through a subzone and have the world map open. It will zoom out back to the zone.",
		function() return DefaultWorldMapZoom.Settings.ZoomOutOfSubZone end, 
		function(value) DefaultWorldMapZoom.Settings.ZoomOutOfSubZone = value end,
		false,
		""
	)]]
end

DefaultWorldMapZoomLocalCallbackManager:RegisterCallback("DefaultWorldMapOnZoneChanged", DefaultWorldMapZoom.OnZoneChanged)

EVENT_MANAGER:RegisterForEvent(DefaultWorldMapZoom.title, EVENT_START_FAST_TRAVEL_INTERACTION , DefaultWorldMapZoom.disableZoom)
EVENT_MANAGER:RegisterForEvent(DefaultWorldMapZoom.title, EVENT_END_FAST_TRAVEL_INTERACTION, DefaultWorldMapZoom.enableZoom)

EVENT_MANAGER:RegisterForEvent(DefaultWorldMapZoom.title, EVENT_ZONE_CHANGED, DefaultWorldMapZoom.showZone)
EVENT_MANAGER:RegisterForEvent(DefaultWorldMapZoom.title, EVENT_ACTION_LAYER_PUSHED, DefaultWorldMapZoom.Start)
EVENT_MANAGER:RegisterForEvent(DefaultWorldMapZoom.title, EVENT_ACTION_LAYER_POPPED, DefaultWorldMapZoom.End)
EVENT_MANAGER:RegisterForEvent(DefaultWorldMapZoom.title, EVENT_ADD_ON_LOADED, DefaultWorldMapZoom.Initialize)